@@ -1,7 +1,7 @@
 /**
  * UGENE - Integrated Bioinformatics Tools.
- * Copyright (C) 2008-2013 UniPro <ugene@unipro.ru>
- * http://ugene.unipro.ru
+ * Copyright (C) 2008-2016 UniPro <ugene@unipro.ru>
+ * http://ugene.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,34 +19,41 @@
  * MA 02110-1301, USA.
  */
 
-#include "SmithWatermanDialogImpl.h"
-
-#include <QtCore/QStringList>
-#include <QtGui/QMessageBox>
-#include <QtGui/QFileDialog>
+#include <QCheckBox>
+#include <QMessageBox>
 #include <QRegExp>
-
-#include <U2Core/AppContext.h>
-#include <U2Core/Log.h>
-#include <U2Core/DNASequenceObject.h>
-#include <U2Gui/CreateAnnotationWidgetController.h>
-#include <U2Remote/RemoteMachineMonitorDialogController.h>
-#include <U2View/ADVSequenceObjectContext.h>
-#include <U2Core/TextUtils.h>
-#include <U2Core/DNASequenceSelection.h>
-#include <U2Gui/DialogUtils.h>
-#include <U2Core/BaseDocumentFormats.h>
-#include <U2Core/ProjectModel.h>
-#include <U2Core/Settings.h>
+#include <QStringList>
 
 #include <U2Algorithm/SmithWatermanReportCallback.h>
+
+#include <U2Core/AppContext.h>
+#include <U2Core/BaseDocumentFormats.h>
+#include <U2Core/DNASequenceObject.h>
+#include <U2Core/DNASequenceSelection.h>
+#include <U2Core/L10n.h>
+#include <U2Core/Log.h>
+#include <U2Core/ProjectModel.h>
+#include <U2Core/Settings.h>
+#include <U2Core/TextUtils.h>
+#include <U2Core/U2OpStatusUtils.h>
+#include <U2Core/U2SafePoints.h>
+
+#include <U2Gui/CreateAnnotationWidgetController.h>
+#include <U2Gui/DialogUtils.h>
+#include <U2Gui/HelpButton.h>
+#include <U2Core/QObjectScopedPointer.h>
+#include <U2Gui/U2FileDialog.h>
+
+#include <U2View/ADVSequenceObjectContext.h>
+
+#include "SmithWatermanDialogImpl.h"
 #include "SubstMatrixDialog.h"
 
 const QString SETTINGS_LASTDIR = "save_align_files/last_dir";
 
-extern const QChar OPEN_SQUARE_BRACKET;
-extern const QChar CLOSE_SQUARE_BRACKET;
-extern const QChar SHORTHAND_AND_LABEL_SEPARATOR;
+extern const char OPEN_SQUARE_BRACKET;
+extern const char CLOSE_SQUARE_BRACKET;
+extern const char SHORTHAND_AND_LABEL_SEPARATOR;
 extern const qint32 STRING_HAS_NO_KEY_MESSAGE;
 
 const quint8 COUNT_OF_TEMPLATE_BUTTONS_IN_ROW = 2;
@@ -58,15 +65,21 @@ const QChar DEFAULT_SHORTHANDS_SEPARATOR = '_';
 
 namespace U2 {
 
-SmithWatermanDialog::SmithWatermanDialog(QWidget* w, 
-                                         ADVSequenceObjectContext* ctx, 
-                                         SWDialogConfig* _dialogConfig):
-    QDialog(w), substMatrixRegistry(0), swTaskFactoryRegistry(0)
+SmithWatermanDialog::SmithWatermanDialog(QWidget* w, ADVSequenceObjectContext* ctx, SWDialogConfig* _dialogConfig)
+    : QDialog(w), substMatrixRegistry(0), swTaskFactoryRegistry(0)
 {
     ctxSeq = ctx;
     dialogConfig = _dialogConfig;
-        
+
     setupUi(this);
+    new HelpButton(this, buttonBox, "18223193");
+    buttonBox->button(QDialogButtonBox::Yes)->setText(tr("Remote run"));
+    buttonBox->button(QDialogButtonBox::Ok)->setText(tr("Search"));
+    buttonBox->button(QDialogButtonBox::Cancel)->setText(tr("Cancel"));
+
+    remoteRunPushButton = buttonBox->button(QDialogButtonBox::Yes);
+    bttnRun = buttonBox->button(QDialogButtonBox::Ok);
+    bttnCancel = buttonBox->button(QDialogButtonBox::Cancel);
 
     tabWidget->setCurrentIndex(0);
     regionSelector = new RegionSelector(this, ctx->getSequenceLength(), true, ctx->getSequenceSelection());
@@ -81,11 +94,11 @@ SmithWatermanDialog::SmithWatermanDialog(QWidget* w,
         QDialog::done(-1);
         return;
     }
-    
+
     swResultFilterRegistry = AppContext::getSWResultFilterRegistry();
     if (0 == swResultFilterRegistry) {
         coreLog.error(tr("No filter registry found.")); //FIXME should be optional?
-        QDialog::done(-1);                    //No.  
+        QDialog::done(-1);                    //No.
         return;
     }
 
@@ -112,7 +125,7 @@ SmithWatermanDialog::SmithWatermanDialog(QWidget* w,
     updatePatternFieldVisualState();
 
     teditPattern->setFocus();
-    
+
     //! FIXME:
     remoteRunPushButton->setVisible(false);
 }
@@ -208,7 +221,7 @@ bool SmithWatermanDialog::checkTemplateButtonName(const QString & name)
         return false;
 
     quint32 closeBracketIndex = 0;
-    
+
     for(qint32 i = 1; i < name.length(); ++i) {
         if(!name[i].isLetterOrNumber() && !name[i].isSpace() && CLOSE_SQUARE_BRACKET != name[i])
             return false;
@@ -228,7 +241,7 @@ bool SmithWatermanDialog::checkTemplateButtonName(const QString & name)
                 && SHORTHAND_AND_LABEL_SEPARATOR == name[i - 1])
             return true;
     }
-    
+
     return false;
 }
 
@@ -260,7 +273,7 @@ void SmithWatermanDialog::fillTemplateNamesFieldsByDefault()
                                                 .prepend(OPEN_SQUARE_BRACKET);
     patternSubseqNameTmpl->setText(defaultPtrnSubseqTagsString);
     delete defaultPtrnSubseqTags;
-    
+
     patternSequenceName->setText(DEFAULT_PATTERN_SEQUENCE_NAME + QString::number(dialogConfig->countOfLaunchesAlgorithm));
 }
 
@@ -295,32 +308,38 @@ void SmithWatermanDialog::fillTemplateButtonsList()
 
 void SmithWatermanDialog::addAnnotationWidget()
 {
-
     U2SequenceObject *dnaso = qobject_cast<U2SequenceObject*>(ctxSeq->getSequenceGObject());
     CreateAnnotationModel acm;
 
     acm.sequenceObjectRef = GObjectReference(dnaso);
-    acm.hideAnnotationName = false;
     acm.hideLocation = true;
+    acm.useAminoAnnotationTypes = ctxSeq->getAlphabet()->isAmino();
     acm.sequenceLen = dnaso->getSequenceLength();
     annotationController = new CreateAnnotationWidgetController(acm, this);
-    QWidget* caw = annotationController->getWidget();    
+    QWidget* caw = annotationController->getWidget();
     QVBoxLayout* l = new QVBoxLayout();
     l->setMargin(0);
     l->addWidget(caw);
+
+    addPatternContentQualifier = new QCheckBox(
+        tr( "Add qualifier with corresponding pattern subsequences to result annotations" ),
+        annotationsWidget );
+    addPatternContentQualifier->setObjectName("addPatternContentQualifierCheck");
+    l->addWidget( addPatternContentQualifier );
+
     annotationsWidget->setLayout(l);
     annotationsWidget->setMinimumSize(caw->layout()->minimumSize());
-} 
+}
 
 void SmithWatermanDialog::setParameters()
 {
-    DNAAlphabet* alphabet = ctxSeq->getAlphabet();
+    const DNAAlphabet* alphabet = ctxSeq->getAlphabet();
     QStringList matrixList = substMatrixRegistry->selectMatrixNamesByAlphabet(alphabet);
     if (!matrixList.isEmpty())
         bttnViewMatrix->setEnabled(true);
 
     comboMatrix->addItems(matrixList);
-    
+
     QStringList alg_lst = swTaskFactoryRegistry->getListFactoryNames();
     comboRealization->addItems(alg_lst);
 
@@ -333,7 +352,7 @@ void SmithWatermanDialog::setParameters()
     radioSequence->setChecked(true);
     if (0 != ctxSeq->getAminoTT())
         radioTranslation->setEnabled(true);
-    
+
     radioDirect->setEnabled(true);
     if (ctxSeq->getComplementTT() != NULL) {
         radioComplement->setEnabled(true);
@@ -349,7 +368,7 @@ void SmithWatermanDialog::setParameters()
     resultViewVariants->addItem(tr(SmithWatermanSettings::getResultViewNames()[SmithWatermanSettings::MULTIPLE_ALIGNMENT]));
     resultViewVariants->addItem(tr(SmithWatermanSettings::getResultViewNames()[SmithWatermanSettings::ANNOTATIONS]));
     resultViewVariants->setCurrentIndex(0);
-    
+
     mObjectNameTmpl->installEventFilter(this);
     refSubseqNameTmpl->installEventFilter(this);
     patternSubseqNameTmpl->installEventFilter(this);
@@ -398,7 +417,7 @@ void SmithWatermanDialog::templateEditLostFocus()
 void SmithWatermanDialog::sl_browseAlignFilesDir()
 {
     const QString openUrl = QFileInfo(alignmentFilesPath->text()).absoluteDir().absolutePath();
-    const QString name = QFileDialog::getExistingDirectory(this, tr("Choose folder"), openUrl);
+    const QString name = U2FileDialog::getExistingDirectory(this, tr("Choose folder"), openUrl);
     if (!name.isEmpty()) {
         alignmentFilesPath->setText(name + '/');
         AppContext::getSettings()->setValue(SETTINGS_LASTDIR, name, true);
@@ -413,13 +432,13 @@ void SmithWatermanDialog::sl_bttnViewMatrix()
         QMessageBox::critical(this, windowTitle(), tr("Matrix not found."));
         return;
     }
-    SubstMatrixDialog smDialog(mtx, this);
-    smDialog.exec();
+    QObjectScopedPointer<SubstMatrixDialog> smDialog = new SubstMatrixDialog(mtx, this);
+    smDialog->exec();
 }
 
 void SmithWatermanDialog::sl_translationToggled(bool checked)
 {
-    DNAAlphabet* alphabet = 0;
+    const DNAAlphabet* alphabet = 0;
     if (checked) {
         DNATranslation* aminoTT = ctxSeq->getAminoTT();
         alphabet = aminoTT->getDstAlphabet();
@@ -454,56 +473,66 @@ void SmithWatermanDialog::sl_bttnRun()
         err = validateResultDirPath();
 
     if (!err.isEmpty()) {
-        QMessageBox::critical(this, tr("Error!"), err);
+        QMessageBox::critical(this, L10N::errorTitle(), err);
         return;
     }
     if (readParameters()) {
         if(SmithWatermanSettings::ANNOTATIONS == config.resultView) {
             bool objectPrepared = annotationController->prepareAnnotationObject();
             if (!objectPrepared){
-                QMessageBox::critical(this, tr("Error!"), "Cannot create an annotation object. Please check settings");
+                QMessageBox::critical(this, L10N::errorTitle(), tr("Cannot create an annotation object. Please check settings."));
                 return;
             }
             const CreateAnnotationModel& m = annotationController->getModel();
-            AnnotationTableObject* obj = m.getAnnotationObject();
+            AnnotationTableObject *obj = m.getAnnotationObject();
+            U2FeatureType annotationType = m.data->type;
             QString annotationName = m.data->name;
             QString annotationGroup = m.groupName;
-        
-            config.resultCallback = new SmithWatermanReportCallbackAnnotImpl(obj, annotationName, annotationGroup);
-        } else if (SmithWatermanSettings::MULTIPLE_ALIGNMENT == config.resultView){
+
+            config.resultCallback = new SmithWatermanReportCallbackAnnotImpl(obj, annotationType, annotationName,
+                annotationGroup, m.description, addPatternContentQualifier->isChecked());
+            config.includePatternContent = addPatternContentQualifier->isChecked();
+        } else if (SmithWatermanSettings::MULTIPLE_ALIGNMENT == config.resultView) {
             const U2SequenceObject * sequence = ctxSeq->getSequenceObject();
+            U2OpStatusImpl os;
+            QByteArray seqData = sequence->getWholeSequenceData(os);
+            CHECK_OP_EXT(os, QMessageBox::critical(this, L10N::errorTitle(), os.getError()), );
             config.resultCallback = new SmithWatermanReportCallbackMAImpl(alignmentFilesPath->text(), mObjectNameTmpl->text(),
                                                                         refSubseqNameTmpl->text(), patternSubseqNameTmpl->text(),
-                                                                        sequence->getWholeSequenceData(), config.ptrn,
+                                                                        seqData, config.ptrn,
                                                                         sequence->getSequenceName(), patternSequenceName->text(),
                                                                         sequence->getAlphabet());
         }
         config.resultListener = new SmithWatermanResultListener;
-        
-        Task* task = realization->getTaskInstance(config, tr("SmithWatermanTask") );            
+
+        Task* task = realization->getTaskInstance(config, tr("SmithWatermanTask") );
         AppContext::getTaskScheduler()->registerTopLevelTask(task);
         saveDialogConfig();
         QDialog::accept();
-    } else
+    } else {
         clearAll();
+    }
 }
 
 bool SmithWatermanDialog::readParameters()
 {
     clearAll();
 
-    config.sqnc = ctxSeq->getSequenceObject()->getWholeSequenceData();
-    
+    U2OpStatusImpl os;
+    config.sqnc = ctxSeq->getSequenceObject()->getWholeSequenceData(os);
+    CHECK_OP_EXT(os, QMessageBox::critical(this, L10N::errorTitle(), os.getError()), false);
+    config.searchCircular = ctxSeq->getSequenceObject()->isCircular();
+
     DNATranslation* aminoTT = 0;
     bool isTranslation = radioTranslation->isChecked();
     if (isTranslation)
         aminoTT = ctxSeq->getAminoTT();
     else
         aminoTT = 0;
-    
+
     if (!readSubstMatrix())
         return false;
-    
+
     if (!readPattern(aminoTT))
         return false;
 
@@ -511,7 +540,7 @@ bool SmithWatermanDialog::readParameters()
 
     if (!readRegion() || !readGapModel() || !readResultFilter() || !readRealization())
         return false;
-    
+
     qint32 resultView = config.getResultViewKeyForString(resultViewVariants->currentText());
     if(STRING_HAS_NO_KEY_MESSAGE != resultView) {
         config.resultView = static_cast<SmithWatermanSettings::SWResultView>(resultView);
@@ -525,7 +554,7 @@ bool SmithWatermanDialog::readParameters()
         }
     } else
         assert(0);
-    
+
     if (radioDirect->isChecked())
         config.strand = StrandOption_DirectOnly;
     else if (radioComplement->isChecked())
@@ -540,7 +569,7 @@ bool SmithWatermanDialog::readParameters()
         QMessageBox::critical(this, windowTitle(), tr("Complement translation is not found."));
         return false;
     }
-        
+
     return true;
 }
 
@@ -561,7 +590,7 @@ bool SmithWatermanDialog::readSubstMatrix()
     QString strSelectedMatrix = comboMatrix->currentText();
     SMatrix mtx = substMatrixRegistry->getMatrix(strSelectedMatrix);
     if (mtx.isEmpty()) {
-        QMessageBox::critical(this, windowTitle(), 
+        QMessageBox::critical(this, windowTitle(),
                               tr("Matrix %1 is not found.").arg(strSelectedMatrix));
         return false;
     }
@@ -580,7 +609,7 @@ bool SmithWatermanDialog::readGapModel()
 {
     float scoreGapOpen = spinGapOpen->value();
     config.gapModel.scoreGapOpen = scoreGapOpen;
-    
+
     float scoreGapExtd = spinGapExtd->value();
     config.gapModel.scoreGapExtd = scoreGapExtd;
 
@@ -591,7 +620,7 @@ bool SmithWatermanDialog::readResultFilter()
 {
     int percentOfScore = spinScorePercent->value();
     config.percentOfScore = percentOfScore;
-    
+
     QString strSelectedFilter = comboResultFilter->currentText();
     SmithWatermanResultFilter* filter = swResultFilterRegistry->getFilter(strSelectedFilter);
     if (0 == filter) {
@@ -604,10 +633,10 @@ bool SmithWatermanDialog::readResultFilter()
 
 bool SmithWatermanDialog::readPattern(DNATranslation* aminoTT)
 {
-    DNAAlphabet* al = 0;
+    const DNAAlphabet* al = 0;
     if (0 == aminoTT) {
         assert(config.pSm.getAlphabet() != NULL);
-        al = config.pSm.getAlphabet(); 
+        al = config.pSm.getAlphabet();
     } else
         al = aminoTT->getDstAlphabet();
 
@@ -623,14 +652,14 @@ bool SmithWatermanDialog::readPattern(DNATranslation* aminoTT)
         QMessageBox::critical(this, windowTitle(),  tr("Pattern is empty"));
         return false;
     }
-        
+
     QByteArray pattern;
     if (!al->isCaseSensitive()) {
         QString upperPattern = inputPattern.toUpper();
         pattern = upperPattern.toLocal8Bit();
     } else
         pattern = inputPattern.toLocal8Bit();
-    
+
     if (!TextUtils::fits(al->getMap(), pattern.constData(), pattern.length())) {
         QMessageBox::critical(this, windowTitle(),  tr("Pattern contains unknown symbol"));
         return false;
@@ -669,7 +698,7 @@ void SmithWatermanDialog::clearAll()
     config.strand = StrandOption_DirectOnly;
     config.complTT = 0;
     config.aminoTT = 0;
-    
+
     realization = 0;
 }
 
@@ -683,8 +712,10 @@ void SmithWatermanDialog::loadDialogConfig()
     case (SmithWatermanSearchType_inTranslation):
         radioTranslation->setChecked(true);
         break;
+    default:
+        break;
     }
-    
+
     const StrandOption strand = dialogConfig->strand;
     switch (strand) {
     case (StrandOption_DirectOnly):
@@ -700,21 +731,6 @@ void SmithWatermanDialog::loadDialogConfig()
         assert(0);
     }
 
-//    const SmithWatermanRangeType rangeType = dialogConfig->rangeType;
-//    switch (rangeType) {
-//    case (SmithWatermanRangeType_wholeSequence):
-//        radioWholeSequence->setChecked(true);
-//        break;
-//    case (SmithWatermanRangeType_selectedRange):
-//        radioSelectedRange->setChecked(true);
-//        break;
-//    case (SmithWatermanRangeType_customRange):
-//        radioCustomRange->setChecked(true);
-//        break;
-//    default:
-//        break;
-//    }
-
     const QByteArray& prevPattern = dialogConfig->ptrn;
     if (!prevPattern.isEmpty())
         teditPattern->setText(prevPattern);
@@ -745,7 +761,7 @@ void SmithWatermanDialog::loadDialogConfig()
         assert(-1 != filterIndex);
         comboResultFilter->setCurrentIndex(filterIndex);
     }
-        
+
     const float minScoreInPercent = dialogConfig->minScoreInPercent;
     spinScorePercent->setValue(minScoreInPercent);
 
@@ -771,7 +787,8 @@ void SmithWatermanDialog::loadDialogConfig()
         patternSequenceName->text().isEmpty())
         fillTemplateNamesFieldsByDefault();
 
-	advOptions->setChecked(dialogConfig->enableAdvancedMASettings);
+    advOptions->setChecked(dialogConfig->enableAdvancedMASettings);
+    addPatternContentQualifier->setChecked( dialogConfig->addPatternSubsequenceAsQualifier );
 
     return;
 }
@@ -779,39 +796,30 @@ void SmithWatermanDialog::loadDialogConfig()
 void SmithWatermanDialog::saveDialogConfig()
 {
     dialogConfig->ptrn = teditPattern->toPlainText().toLatin1();
-    
+
     dialogConfig->algVersion = comboRealization->currentText();
     dialogConfig->scoringMatrix = comboMatrix->currentText();
 
     dialogConfig->gm.scoreGapOpen = (float)spinGapOpen->value();
     dialogConfig->gm.scoreGapExtd = (float)spinGapExtd->value();
-    
+
     dialogConfig->resultFilter = comboResultFilter->currentText();
     dialogConfig->minScoreInPercent = spinScorePercent->value();
-    
-    dialogConfig->searchType =  (radioSequence->isChecked()) ? 
+
+    dialogConfig->searchType =  (radioSequence->isChecked()) ?
                                 (SmithWatermanSearchType_inSequence):
                                 (SmithWatermanSearchType_inTranslation);
 
-    dialogConfig->strand =  (radioDirect->isChecked()) ? 
+    dialogConfig->strand =  (radioDirect->isChecked()) ?
                             (StrandOption_DirectOnly):
                             (dialogConfig->strand);
-    dialogConfig->strand =  (radioComplement->isChecked()) ? 
+    dialogConfig->strand =  (radioComplement->isChecked()) ?
                             (StrandOption_ComplementOnly):
                             (dialogConfig->strand);
-    dialogConfig->strand =  (radioBoth->isChecked()) ? 
+    dialogConfig->strand =  (radioBoth->isChecked()) ?
                             (StrandOption_Both):
                             (dialogConfig->strand);
-    
-//    dialogConfig->rangeType =   (radioWholeSequence->isChecked()) ?
-//                                (SmithWatermanRangeType_wholeSequence):
-//                                (dialogConfig->rangeType);
-//    dialogConfig->rangeType =   (radioSelectedRange->isChecked()) ?
-//                                (SmithWatermanRangeType_selectedRange):
-//                                (dialogConfig->rangeType);
-//    dialogConfig->rangeType =   (radioCustomRange->isChecked()) ?
-//                                (SmithWatermanRangeType_customRange):
-//                                (dialogConfig->rangeType);
+
     const qint32 checkResultView = config.getResultViewKeyForString(resultViewVariants->currentText());
     assert(STRING_HAS_NO_KEY_MESSAGE != checkResultView);
     dialogConfig->resultView = static_cast<SmithWatermanSettings::SWResultView>(checkResultView);
@@ -832,50 +840,20 @@ void SmithWatermanDialog::saveDialogConfig()
     if(!patternName.isEmpty())
         dialogConfig->patternSequenceName = patternName;
 
-	dialogConfig->enableAdvancedMASettings = advOptions->isChecked();
+    dialogConfig->enableAdvancedMASettings = advOptions->isChecked();
 
     if(sender() == bttnRun)
         dialogConfig->countOfLaunchesAlgorithm++;
 
+    dialogConfig->addPatternSubsequenceAsQualifier = addPatternContentQualifier->isChecked( );
+
     return;
 }
 
-//void SmithWatermanDialog::sl_remoteRunButtonClicked() {
-//    // validate annotations controller first
-//    QString err = ac->validate();
-//    if( !err.isEmpty() ) {
-//        QMessageBox::critical( this, tr( "Error!" ), err );
-//        return;
-//    }
-//    if( !readParameters() ) {
-//        clearAll();
-//        return;
-//    }
-//
-//    RemoteMachineMonitor * rmm = AppContext::getRemoteMachineMonitor();
-//    assert(rmm);
-//    RemoteMachineSettings *rms = RemoteMachineMonitorDialogController::selectRemoteMachine(rmm, true);
-//    if (!rms) {
-//        return;
-//    }
-//    assert(rms->getMachineType() == RemoteMachineType_RemoteService);
-//
-//    ac->prepareAnnotationObject();
-//    const CreateAnnotationModel &m = ac->getModel();
-//    
-//    Task *t = new SmithWatermanSchemaRemoteTask(rms, config, m.getAnnotationObject(), m.data->name, m.groupName);
-//    Task *t = new SmithWatermanSchemaTask(config, m.getAnnotationObject(), m.data->name, m.groupName); // for local run
-//    AppContext::getTaskScheduler()->registerTopLevelTask(t);
-//    
-//    saveDialogConfig();
-//    QDialog::accept();
-//}
-
-
 void SmithWatermanDialogController::run(QWidget* p, ADVSequenceObjectContext* ctx, SWDialogConfig* dialogConfig)
 {
-    SmithWatermanDialog smv(p, ctx, dialogConfig);
-    smv.exec();
+    QObjectScopedPointer<SmithWatermanDialog> smv = new SmithWatermanDialog(p, ctx, dialogConfig);
+    smv->exec();
 }
 
 } // namespace
