@@ -1,7 +1,7 @@
 /**
  * UGENE - Integrated Bioinformatics Tools.
- * Copyright (C) 2008-2013 UniPro <ugene@unipro.ru>
- * http://ugene.unipro.ru
+ * Copyright (C) 2008-2016 UniPro <ugene@unipro.ru>
+ * http://ugene.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,7 @@
 #ifndef GET_SEQUENCE_BY_ID_
 #define GET_SEQUENCE_BY_ID_
 
-#include "ui/ui_GetSequenceByIdDialog.h"
+#include "ui_GetSequenceByIdDialog.h"
 
 namespace U2 {
 
@@ -48,4 +48,4 @@ private:
 }
 
 
-#endif
\ No newline at end of file
+#endif
