@@ -1,7 +1,7 @@
 /**
  * UGENE - Integrated Bioinformatics Tools.
- * Copyright (C) 2008-2013 UniPro <ugene@unipro.ru>
- * http://ugene.unipro.ru
+ * Copyright (C) 2008-2016 UniPro <ugene@unipro.ru>
+ * http://ugene.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,11 +30,11 @@ RequiredSlotsValidator::RequiredSlotsValidator(const QList<Descriptor> &_require
 
 }
 
-bool RequiredSlotsValidator::validate(const IntegralBusPort *port, QStringList &errors) const {
+bool RequiredSlotsValidator::validate(const IntegralBusPort *port, ProblemList &problemList) const {
     bool noErrors = true;
     foreach (const Descriptor &d, requiredSlots) {
         if (!isBinded(port, d.getId())) {
-            errors << QObject::tr("Input '%1' slot is not supplied").arg(d.getDisplayName());
+            problemList << Problem(QObject::tr("Input '%1' slot is not supplied").arg(d.getDisplayName()));
             noErrors = false;
         }
     }
